<?php

namespace Drupal\reverse_proxy_header\EventSubscriber;

use Drupal\Core\Site\Settings;
use GuzzleHttp\ClientInterface;
use Psr\Log\LoggerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Restores the true client IP address.
 */
class ReverseProxyHeaderClientIpRestore implements EventSubscriberInterface {

  /**
   * The HTTP client to fetch the feed data with.
   *
   * @var \GuzzleHttp\ClientInterface
   */
  protected $httpClient;

  /**
   * The settings object.
   *
   * @var \Drupal\Core\Site\Settings
   */
  protected $settings;

  /**
   * The logger.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * Constructs a ReverseProxyHeaderClientIpRestore.
   *
   * @param \GuzzleHttp\ClientInterface $http_client
   *   A Guzzle client object.
   * @param \Drupal\Core\Site\Settings $settings
   *   The settings object.
   * @param \Psr\Log\LoggerInterface $logger
   *   The reverse_proxy_header logger channel.
   */
  public function __construct(ClientInterface $http_client, Settings $settings, LoggerInterface $logger) {
    $this->httpClient = $http_client;
    $this->settings = $settings;
    $this->logger = $logger;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    // Priority is set to 350 (to run before RouterListener::onKernelRequest).
    // Which allows AuthenticationSubscriber methods to use the request IP.
    // @see \Symfony\Component\HttpKernel\EventListener\RouterListener::getSubscribedEvents
    // @see \Drupal\Core\EventSubscriber\AuthenticationSubscriber::getSubscribedEvents
    // @see https://www.drupal.org/project/reverse_proxy_header/issues/3290953
    $events[KernelEvents::REQUEST][] = ['onRequest', 350];
    return $events;
  }

  /**
   * Restores the origination client IP from the custom HTTP header.
   */
  public function onRequest(RequestEvent $event) {

    // Check the available settings.
    $reverse_proxy_header_name = $this->settings->get('reverse_proxy_header');
    if (!$reverse_proxy_header_name) {
      return;
    }

    // Check the header value.
    $current_request = $event->getRequest();
    $connecting_ip = $current_request->server->get($reverse_proxy_header_name);
    if ($this->isInvalidIpAddress($connecting_ip, $reverse_proxy_header_name)) {
      return;
    }

    // As the changed remote address will make it impossible to determine
    // a trusted proxy, we need to make sure we set the right protocol as well.
    // @see \Symfony\Component\HttpFoundation\Request::isSecure()
    $event->getRequest()->server->set('HTTPS', $event->getRequest()->isSecure() ? 'on' : 'off');
    $event->getRequest()->server->set('REMOTE_ADDR', $connecting_ip);
    $event->getRequest()->overrideGlobals();
  }

  /**
   * Check if IP address is invalid.
   *
   * @param string $ip_address
   *   Ip address.
   * @param string $header_name
   *   The header name.
   *
   * @return bool
   *   Return TRUE if IP address is invalid.
   */
  protected function isInvalidIpAddress($ip_address, $header_name) {
    $variables = [
      '@ip_address' => $ip_address,
      '@header_name' => $header_name,
    ];

    if (empty($ip_address)) {
      $this->logger->notice('Empty IP address value retrieved from @header_name header is invalid.', $variables);
      return TRUE;
    }

    if (!filter_var($ip_address, FILTER_VALIDATE_IP)) {
      $this->logger->notice('IP address `@ip_address` retrieved from @header_name header is invalid.', $variables);
      return TRUE;
    }

    return FALSE;
  }

}
